@@ -1,8 +1,9 @@
 """
-A 2DPackLib instance.
+A Two-Dimensional Bin Packing instance.
 
 This module provides an instance of the two-dimensional bin packing problem
-as defined in 2DPackLib [1, 2].
+as defined in 2DPackLib [1, 2] as well as the four non-trivial 'Almost Squares
+in Almost Squares' instances [6, 7].
 
 All instances of :class:`~moptipyapps.binpacking2d.instance.Instance`
 are two-dimensional numpy `ndarrays` with additional attributes.
@@ -31,7 +32,9 @@ pre-defined instances as resource which can be obtained via
 :meth:`Instance.from_resource` and listed via
 :meth:`Instance.list_resources`.
 
-We provide the instances of the sets `A` [3], `BENG` [4], and `CLASS` [5].
+We provide the instances of the sets `A` [3], `BENG` [4], and `CLASS` [5]
+from 2DPackLib. Additionally, we include the four non-trivial 'Almost Squares
+in Almost Squares' instances [6,7].
 
 Initial work on this code has been contributed by Mr. Rui ZHAO (赵睿),
 <zr1329142665@163.com> a Master's student at the Institute of Applied
@@ -57,6 +60,17 @@ Prof. Dr. Thomas Weise (汤卫思教授).
 5. J.O. Berkey and P.Y. Wang. Two Dimensional Finite Bin Packing Algorithms.
    *Journal of the Operational Research Society* 38(5):423-429. May 1987.
    https://doi.org/10.1057/jors.1987.70
+6. Daan van den Berg, Florian Braam, Mark Moes, Emiel Suilen, and
+   Sandjai Bhulai. Almost Squares in Almost Squares: Solving the Final
+   Instance. In *The Fifth International Conference on Data Analytics
+   (DATA ANALYTICS'16)* October 9-13, 2016, Venice, Italy, pages 69-74.
+   IARIA. ISBN: 9781612085104. https://hdl.handle.net/11245/1.545914.
+   https://math.vu.nl/~sbhulai/publications/data_analytics2016b.pdf.
+7. H. Simonis and B. O'Sullivan. Almost Square Packing in *8th International
+   Conference on Integration of AI and OR Techniques in Constraint Programming
+   for Combinatorial Optimization Problems* (CPAIOR'11), May 23-27, 2011,
+   Berlin, Germany, pages 196-209. Berlin, Germany: Springer-Verlag.
+   doi:10.1007/978-3-642-21311-3_19.
 
 >>> ins = Instance("a", 100, 50, [[10, 5, 1], [3, 3, 1], [5, 5, 1]])
 >>> ins.name
@@ -100,19 +114,20 @@ dtype('int16')
 
 from importlib import resources  # nosem
 from os.path import basename
-from typing import Final, cast
+from statistics import quantiles
+from typing import Final, Iterable, cast
 
 import moptipy.utils.nputils as npu
 import numpy as np
 from moptipy.api.component import Component
 from moptipy.utils.logger import CSV_SEPARATOR, KeyValueLogSection
 from moptipy.utils.nputils import int_range_to_dtype
-from moptipy.utils.path import Path
 from moptipy.utils.strings import sanitize_name
-from moptipy.utils.types import check_int_range, check_to_int_range, type_error
+from pycommons.io.path import UTF8, Path, file_path
+from pycommons.types import check_int_range, check_to_int_range, type_error
 
 #: the instances resource name
-INSTANCES_RESOURCE: Final[str] = "2dpacklib.txt"
+INSTANCES_RESOURCE: Final[str] = "instances.txt"
 
 #: the total number of items
 N_ITEMS: Final[str] = "nItems"
@@ -132,114 +147,221 @@ IDX_HEIGHT: Final[int] = 1
 #: the index of the repetitions element in an item of an instance
 IDX_REPETITION: Final[int] = 2
 
-#: the list of instance names of the 2DPackLib bin packing set
+#: the list of instance names of the 2DPackLib bin packing set downloaded
+#: from https://site.unibo.it/operations-research/en/research/2dpacklib
+#: ('a*','beng*', 'cl*') as well as the four non-trivial 'Almost Squares in
+#: Almost Squares' instances ('asqas*').
 _INSTANCES: Final[tuple[str, ...]] = (
     "a01", "a02", "a03", "a04", "a05", "a06", "a07", "a08", "a09", "a10",
     "a11", "a12", "a13", "a14", "a15", "a16", "a17", "a18", "a19", "a20",
     "a21", "a22", "a23", "a24", "a25", "a26", "a27", "a28", "a29", "a30",
     "a31", "a32", "a33", "a34", "a35", "a36", "a37", "a38", "a39", "a40",
-    "a41", "a42", "a43", "beng01", "beng02", "beng03", "beng04", "beng05",
-    "beng06", "beng07", "beng08", "beng09", "beng10", "cl01_020_01",
-    "cl01_020_02", "cl01_020_03", "cl01_020_04", "cl01_020_05", "cl01_020_06",
-    "cl01_020_07", "cl01_020_08", "cl01_020_09", "cl01_020_10", "cl01_040_01",
-    "cl01_040_02", "cl01_040_03", "cl01_040_04", "cl01_040_05", "cl01_040_06",
-    "cl01_040_07", "cl01_040_08", "cl01_040_09", "cl01_040_10", "cl01_060_01",
-    "cl01_060_02", "cl01_060_03", "cl01_060_04", "cl01_060_05", "cl01_060_06",
-    "cl01_060_07", "cl01_060_08", "cl01_060_09", "cl01_060_10", "cl01_080_01",
-    "cl01_080_02", "cl01_080_03", "cl01_080_04", "cl01_080_05", "cl01_080_06",
-    "cl01_080_07", "cl01_080_08", "cl01_080_09", "cl01_080_10", "cl01_100_01",
-    "cl01_100_02", "cl01_100_03", "cl01_100_04", "cl01_100_05", "cl01_100_06",
-    "cl01_100_07", "cl01_100_08", "cl01_100_09", "cl01_100_10", "cl02_020_01",
-    "cl02_020_02", "cl02_020_03", "cl02_020_04", "cl02_020_05", "cl02_020_06",
-    "cl02_020_07", "cl02_020_08", "cl02_020_09", "cl02_020_10", "cl02_040_01",
-    "cl02_040_02", "cl02_040_03", "cl02_040_04", "cl02_040_05", "cl02_040_06",
-    "cl02_040_07", "cl02_040_08", "cl02_040_09", "cl02_040_10", "cl02_060_01",
-    "cl02_060_02", "cl02_060_03", "cl02_060_04", "cl02_060_05", "cl02_060_06",
-    "cl02_060_07", "cl02_060_08", "cl02_060_09", "cl02_060_10", "cl02_080_01",
-    "cl02_080_02", "cl02_080_03", "cl02_080_04", "cl02_080_05", "cl02_080_06",
-    "cl02_080_07", "cl02_080_08", "cl02_080_09", "cl02_080_10", "cl02_100_01",
-    "cl02_100_02", "cl02_100_03", "cl02_100_04", "cl02_100_05", "cl02_100_06",
-    "cl02_100_07", "cl02_100_08", "cl02_100_09", "cl02_100_10", "cl03_020_01",
-    "cl03_020_02", "cl03_020_03", "cl03_020_04", "cl03_020_05", "cl03_020_06",
-    "cl03_020_07", "cl03_020_08", "cl03_020_09", "cl03_020_10", "cl03_040_01",
-    "cl03_040_02", "cl03_040_03", "cl03_040_04", "cl03_040_05", "cl03_040_06",
-    "cl03_040_07", "cl03_040_08", "cl03_040_09", "cl03_040_10", "cl03_060_01",
-    "cl03_060_02", "cl03_060_03", "cl03_060_04", "cl03_060_05", "cl03_060_06",
-    "cl03_060_07", "cl03_060_08", "cl03_060_09", "cl03_060_10", "cl03_080_01",
-    "cl03_080_02", "cl03_080_03", "cl03_080_04", "cl03_080_05", "cl03_080_06",
-    "cl03_080_07", "cl03_080_08", "cl03_080_09", "cl03_080_10", "cl03_100_01",
-    "cl03_100_02", "cl03_100_03", "cl03_100_04", "cl03_100_05", "cl03_100_06",
-    "cl03_100_07", "cl03_100_08", "cl03_100_09", "cl03_100_10", "cl04_020_01",
-    "cl04_020_02", "cl04_020_03", "cl04_020_04", "cl04_020_05", "cl04_020_06",
-    "cl04_020_07", "cl04_020_08", "cl04_020_09", "cl04_020_10", "cl04_040_01",
-    "cl04_040_02", "cl04_040_03", "cl04_040_04", "cl04_040_05", "cl04_040_06",
-    "cl04_040_07", "cl04_040_08", "cl04_040_09", "cl04_040_10", "cl04_060_01",
-    "cl04_060_02", "cl04_060_03", "cl04_060_04", "cl04_060_05", "cl04_060_06",
-    "cl04_060_07", "cl04_060_08", "cl04_060_09", "cl04_060_10", "cl04_080_01",
-    "cl04_080_02", "cl04_080_03", "cl04_080_04", "cl04_080_05", "cl04_080_06",
-    "cl04_080_07", "cl04_080_08", "cl04_080_09", "cl04_080_10", "cl04_100_01",
-    "cl04_100_02", "cl04_100_03", "cl04_100_04", "cl04_100_05", "cl04_100_06",
-    "cl04_100_07", "cl04_100_08", "cl04_100_09", "cl04_100_10", "cl05_020_01",
-    "cl05_020_02", "cl05_020_03", "cl05_020_04", "cl05_020_05", "cl05_020_06",
-    "cl05_020_07", "cl05_020_08", "cl05_020_09", "cl05_020_10", "cl05_040_01",
-    "cl05_040_02", "cl05_040_03", "cl05_040_04", "cl05_040_05", "cl05_040_06",
-    "cl05_040_07", "cl05_040_08", "cl05_040_09", "cl05_040_10", "cl05_060_01",
-    "cl05_060_02", "cl05_060_03", "cl05_060_04", "cl05_060_05", "cl05_060_06",
-    "cl05_060_07", "cl05_060_08", "cl05_060_09", "cl05_060_10", "cl05_080_01",
-    "cl05_080_02", "cl05_080_03", "cl05_080_04", "cl05_080_05", "cl05_080_06",
-    "cl05_080_07", "cl05_080_08", "cl05_080_09", "cl05_080_10", "cl05_100_01",
-    "cl05_100_02", "cl05_100_03", "cl05_100_04", "cl05_100_05", "cl05_100_06",
-    "cl05_100_07", "cl05_100_08", "cl05_100_09", "cl05_100_10", "cl06_020_01",
-    "cl06_020_02", "cl06_020_03", "cl06_020_04", "cl06_020_05", "cl06_020_06",
-    "cl06_020_07", "cl06_020_08", "cl06_020_09", "cl06_020_10", "cl06_040_01",
-    "cl06_040_02", "cl06_040_03", "cl06_040_04", "cl06_040_05", "cl06_040_06",
-    "cl06_040_07", "cl06_040_08", "cl06_040_09", "cl06_040_10", "cl06_060_01",
-    "cl06_060_02", "cl06_060_03", "cl06_060_04", "cl06_060_05", "cl06_060_06",
-    "cl06_060_07", "cl06_060_08", "cl06_060_09", "cl06_060_10", "cl06_080_01",
-    "cl06_080_02", "cl06_080_03", "cl06_080_04", "cl06_080_05", "cl06_080_06",
-    "cl06_080_07", "cl06_080_08", "cl06_080_09", "cl06_080_10", "cl06_100_01",
-    "cl06_100_02", "cl06_100_03", "cl06_100_04", "cl06_100_05", "cl06_100_06",
-    "cl06_100_07", "cl06_100_08", "cl06_100_09", "cl06_100_10", "cl07_020_01",
-    "cl07_020_02", "cl07_020_03", "cl07_020_04", "cl07_020_05", "cl07_020_06",
-    "cl07_020_07", "cl07_020_08", "cl07_020_09", "cl07_020_10", "cl07_040_01",
-    "cl07_040_02", "cl07_040_03", "cl07_040_04", "cl07_040_05", "cl07_040_06",
-    "cl07_040_07", "cl07_040_08", "cl07_040_09", "cl07_040_10", "cl07_060_01",
-    "cl07_060_02", "cl07_060_03", "cl07_060_04", "cl07_060_05", "cl07_060_06",
-    "cl07_060_07", "cl07_060_08", "cl07_060_09", "cl07_060_10", "cl07_080_01",
-    "cl07_080_02", "cl07_080_03", "cl07_080_04", "cl07_080_05", "cl07_080_06",
-    "cl07_080_07", "cl07_080_08", "cl07_080_09", "cl07_080_10", "cl07_100_01",
-    "cl07_100_02", "cl07_100_03", "cl07_100_04", "cl07_100_05", "cl07_100_06",
-    "cl07_100_07", "cl07_100_08", "cl07_100_09", "cl07_100_10", "cl08_020_01",
-    "cl08_020_02", "cl08_020_03", "cl08_020_04", "cl08_020_05", "cl08_020_06",
-    "cl08_020_07", "cl08_020_08", "cl08_020_09", "cl08_020_10", "cl08_040_01",
-    "cl08_040_02", "cl08_040_03", "cl08_040_04", "cl08_040_05", "cl08_040_06",
-    "cl08_040_07", "cl08_040_08", "cl08_040_09", "cl08_040_10", "cl08_060_01",
-    "cl08_060_02", "cl08_060_03", "cl08_060_04", "cl08_060_05", "cl08_060_06",
-    "cl08_060_07", "cl08_060_08", "cl08_060_09", "cl08_060_10", "cl08_080_01",
-    "cl08_080_02", "cl08_080_03", "cl08_080_04", "cl08_080_05", "cl08_080_06",
-    "cl08_080_07", "cl08_080_08", "cl08_080_09", "cl08_080_10", "cl08_100_01",
-    "cl08_100_02", "cl08_100_03", "cl08_100_04", "cl08_100_05", "cl08_100_06",
-    "cl08_100_07", "cl08_100_08", "cl08_100_09", "cl08_100_10", "cl09_020_01",
-    "cl09_020_02", "cl09_020_03", "cl09_020_04", "cl09_020_05", "cl09_020_06",
-    "cl09_020_07", "cl09_020_08", "cl09_020_09", "cl09_020_10", "cl09_040_01",
-    "cl09_040_02", "cl09_040_03", "cl09_040_04", "cl09_040_05", "cl09_040_06",
-    "cl09_040_07", "cl09_040_08", "cl09_040_09", "cl09_040_10", "cl09_060_01",
-    "cl09_060_02", "cl09_060_03", "cl09_060_04", "cl09_060_05", "cl09_060_06",
-    "cl09_060_07", "cl09_060_08", "cl09_060_09", "cl09_060_10", "cl09_080_01",
-    "cl09_080_02", "cl09_080_03", "cl09_080_04", "cl09_080_05", "cl09_080_06",
-    "cl09_080_07", "cl09_080_08", "cl09_080_09", "cl09_080_10", "cl09_100_01",
-    "cl09_100_02", "cl09_100_03", "cl09_100_04", "cl09_100_05", "cl09_100_06",
-    "cl09_100_07", "cl09_100_08", "cl09_100_09", "cl09_100_10", "cl10_020_01",
-    "cl10_020_02", "cl10_020_03", "cl10_020_04", "cl10_020_05", "cl10_020_06",
-    "cl10_020_07", "cl10_020_08", "cl10_020_09", "cl10_020_10", "cl10_040_01",
-    "cl10_040_02", "cl10_040_03", "cl10_040_04", "cl10_040_05", "cl10_040_06",
-    "cl10_040_07", "cl10_040_08", "cl10_040_09", "cl10_040_10", "cl10_060_01",
-    "cl10_060_02", "cl10_060_03", "cl10_060_04", "cl10_060_05", "cl10_060_06",
-    "cl10_060_07", "cl10_060_08", "cl10_060_09", "cl10_060_10", "cl10_080_01",
-    "cl10_080_02", "cl10_080_03", "cl10_080_04", "cl10_080_05", "cl10_080_06",
-    "cl10_080_07", "cl10_080_08", "cl10_080_09", "cl10_080_10", "cl10_100_01",
-    "cl10_100_02", "cl10_100_03", "cl10_100_04", "cl10_100_05", "cl10_100_06",
-    "cl10_100_07", "cl10_100_08", "cl10_100_09", "cl10_100_10")
+    "a41", "a42", "a43", "asqas03", "asqas08", "asqas20", "asqas34", "beng01",
+    "beng02", "beng03", "beng04", "beng05", "beng06", "beng07", "beng08",
+    "beng09", "beng10", "cl01_020_01", "cl01_020_02", "cl01_020_03",
+    "cl01_020_04", "cl01_020_05", "cl01_020_06", "cl01_020_07", "cl01_020_08",
+    "cl01_020_09", "cl01_020_10", "cl01_040_01", "cl01_040_02", "cl01_040_03",
+    "cl01_040_04", "cl01_040_05", "cl01_040_06", "cl01_040_07", "cl01_040_08",
+    "cl01_040_09", "cl01_040_10", "cl01_060_01", "cl01_060_02", "cl01_060_03",
+    "cl01_060_04", "cl01_060_05", "cl01_060_06", "cl01_060_07", "cl01_060_08",
+    "cl01_060_09", "cl01_060_10", "cl01_080_01", "cl01_080_02", "cl01_080_03",
+    "cl01_080_04", "cl01_080_05", "cl01_080_06", "cl01_080_07", "cl01_080_08",
+    "cl01_080_09", "cl01_080_10", "cl01_100_01", "cl01_100_02", "cl01_100_03",
+    "cl01_100_04", "cl01_100_05", "cl01_100_06", "cl01_100_07", "cl01_100_08",
+    "cl01_100_09", "cl01_100_10", "cl02_020_01", "cl02_020_02", "cl02_020_03",
+    "cl02_020_04", "cl02_020_05", "cl02_020_06", "cl02_020_07", "cl02_020_08",
+    "cl02_020_09", "cl02_020_10", "cl02_040_01", "cl02_040_02", "cl02_040_03",
+    "cl02_040_04", "cl02_040_05", "cl02_040_06", "cl02_040_07", "cl02_040_08",
+    "cl02_040_09", "cl02_040_10", "cl02_060_01", "cl02_060_02", "cl02_060_03",
+    "cl02_060_04", "cl02_060_05", "cl02_060_06", "cl02_060_07", "cl02_060_08",
+    "cl02_060_09", "cl02_060_10", "cl02_080_01", "cl02_080_02", "cl02_080_03",
+    "cl02_080_04", "cl02_080_05", "cl02_080_06", "cl02_080_07", "cl02_080_08",
+    "cl02_080_09", "cl02_080_10", "cl02_100_01", "cl02_100_02", "cl02_100_03",
+    "cl02_100_04", "cl02_100_05", "cl02_100_06", "cl02_100_07", "cl02_100_08",
+    "cl02_100_09", "cl02_100_10", "cl03_020_01", "cl03_020_02", "cl03_020_03",
+    "cl03_020_04", "cl03_020_05", "cl03_020_06", "cl03_020_07", "cl03_020_08",
+    "cl03_020_09", "cl03_020_10", "cl03_040_01", "cl03_040_02", "cl03_040_03",
+    "cl03_040_04", "cl03_040_05", "cl03_040_06", "cl03_040_07", "cl03_040_08",
+    "cl03_040_09", "cl03_040_10", "cl03_060_01", "cl03_060_02", "cl03_060_03",
+    "cl03_060_04", "cl03_060_05", "cl03_060_06", "cl03_060_07", "cl03_060_08",
+    "cl03_060_09", "cl03_060_10", "cl03_080_01", "cl03_080_02", "cl03_080_03",
+    "cl03_080_04", "cl03_080_05", "cl03_080_06", "cl03_080_07", "cl03_080_08",
+    "cl03_080_09", "cl03_080_10", "cl03_100_01", "cl03_100_02", "cl03_100_03",
+    "cl03_100_04", "cl03_100_05", "cl03_100_06", "cl03_100_07", "cl03_100_08",
+    "cl03_100_09", "cl03_100_10", "cl04_020_01", "cl04_020_02", "cl04_020_03",
+    "cl04_020_04", "cl04_020_05", "cl04_020_06", "cl04_020_07", "cl04_020_08",
+    "cl04_020_09", "cl04_020_10", "cl04_040_01", "cl04_040_02", "cl04_040_03",
+    "cl04_040_04", "cl04_040_05", "cl04_040_06", "cl04_040_07", "cl04_040_08",
+    "cl04_040_09", "cl04_040_10", "cl04_060_01", "cl04_060_02", "cl04_060_03",
+    "cl04_060_04", "cl04_060_05", "cl04_060_06", "cl04_060_07", "cl04_060_08",
+    "cl04_060_09", "cl04_060_10", "cl04_080_01", "cl04_080_02", "cl04_080_03",
+    "cl04_080_04", "cl04_080_05", "cl04_080_06", "cl04_080_07", "cl04_080_08",
+    "cl04_080_09", "cl04_080_10", "cl04_100_01", "cl04_100_02", "cl04_100_03",
+    "cl04_100_04", "cl04_100_05", "cl04_100_06", "cl04_100_07", "cl04_100_08",
+    "cl04_100_09", "cl04_100_10", "cl05_020_01", "cl05_020_02", "cl05_020_03",
+    "cl05_020_04", "cl05_020_05", "cl05_020_06", "cl05_020_07", "cl05_020_08",
+    "cl05_020_09", "cl05_020_10", "cl05_040_01", "cl05_040_02", "cl05_040_03",
+    "cl05_040_04", "cl05_040_05", "cl05_040_06", "cl05_040_07", "cl05_040_08",
+    "cl05_040_09", "cl05_040_10", "cl05_060_01", "cl05_060_02", "cl05_060_03",
+    "cl05_060_04", "cl05_060_05", "cl05_060_06", "cl05_060_07", "cl05_060_08",
+    "cl05_060_09", "cl05_060_10", "cl05_080_01", "cl05_080_02", "cl05_080_03",
+    "cl05_080_04", "cl05_080_05", "cl05_080_06", "cl05_080_07", "cl05_080_08",
+    "cl05_080_09", "cl05_080_10", "cl05_100_01", "cl05_100_02", "cl05_100_03",
+    "cl05_100_04", "cl05_100_05", "cl05_100_06", "cl05_100_07", "cl05_100_08",
+    "cl05_100_09", "cl05_100_10", "cl06_020_01", "cl06_020_02", "cl06_020_03",
+    "cl06_020_04", "cl06_020_05", "cl06_020_06", "cl06_020_07", "cl06_020_08",
+    "cl06_020_09", "cl06_020_10", "cl06_040_01", "cl06_040_02", "cl06_040_03",
+    "cl06_040_04", "cl06_040_05", "cl06_040_06", "cl06_040_07", "cl06_040_08",
+    "cl06_040_09", "cl06_040_10", "cl06_060_01", "cl06_060_02", "cl06_060_03",
+    "cl06_060_04", "cl06_060_05", "cl06_060_06", "cl06_060_07", "cl06_060_08",
+    "cl06_060_09", "cl06_060_10", "cl06_080_01", "cl06_080_02", "cl06_080_03",
+    "cl06_080_04", "cl06_080_05", "cl06_080_06", "cl06_080_07", "cl06_080_08",
+    "cl06_080_09", "cl06_080_10", "cl06_100_01", "cl06_100_02", "cl06_100_03",
+    "cl06_100_04", "cl06_100_05", "cl06_100_06", "cl06_100_07", "cl06_100_08",
+    "cl06_100_09", "cl06_100_10", "cl07_020_01", "cl07_020_02", "cl07_020_03",
+    "cl07_020_04", "cl07_020_05", "cl07_020_06", "cl07_020_07", "cl07_020_08",
+    "cl07_020_09", "cl07_020_10", "cl07_040_01", "cl07_040_02", "cl07_040_03",
+    "cl07_040_04", "cl07_040_05", "cl07_040_06", "cl07_040_07", "cl07_040_08",
+    "cl07_040_09", "cl07_040_10", "cl07_060_01", "cl07_060_02", "cl07_060_03",
+    "cl07_060_04", "cl07_060_05", "cl07_060_06", "cl07_060_07", "cl07_060_08",
+    "cl07_060_09", "cl07_060_10", "cl07_080_01", "cl07_080_02", "cl07_080_03",
+    "cl07_080_04", "cl07_080_05", "cl07_080_06", "cl07_080_07", "cl07_080_08",
+    "cl07_080_09", "cl07_080_10", "cl07_100_01", "cl07_100_02", "cl07_100_03",
+    "cl07_100_04", "cl07_100_05", "cl07_100_06", "cl07_100_07", "cl07_100_08",
+    "cl07_100_09", "cl07_100_10", "cl08_020_01", "cl08_020_02", "cl08_020_03",
+    "cl08_020_04", "cl08_020_05", "cl08_020_06", "cl08_020_07", "cl08_020_08",
+    "cl08_020_09", "cl08_020_10", "cl08_040_01", "cl08_040_02", "cl08_040_03",
+    "cl08_040_04", "cl08_040_05", "cl08_040_06", "cl08_040_07", "cl08_040_08",
+    "cl08_040_09", "cl08_040_10", "cl08_060_01", "cl08_060_02", "cl08_060_03",
+    "cl08_060_04", "cl08_060_05", "cl08_060_06", "cl08_060_07", "cl08_060_08",
+    "cl08_060_09", "cl08_060_10", "cl08_080_01", "cl08_080_02", "cl08_080_03",
+    "cl08_080_04", "cl08_080_05", "cl08_080_06", "cl08_080_07", "cl08_080_08",
+    "cl08_080_09", "cl08_080_10", "cl08_100_01", "cl08_100_02", "cl08_100_03",
+    "cl08_100_04", "cl08_100_05", "cl08_100_06", "cl08_100_07", "cl08_100_08",
+    "cl08_100_09", "cl08_100_10", "cl09_020_01", "cl09_020_02", "cl09_020_03",
+    "cl09_020_04", "cl09_020_05", "cl09_020_06", "cl09_020_07", "cl09_020_08",
+    "cl09_020_09", "cl09_020_10", "cl09_040_01", "cl09_040_02", "cl09_040_03",
+    "cl09_040_04", "cl09_040_05", "cl09_040_06", "cl09_040_07", "cl09_040_08",
+    "cl09_040_09", "cl09_040_10", "cl09_060_01", "cl09_060_02", "cl09_060_03",
+    "cl09_060_04", "cl09_060_05", "cl09_060_06", "cl09_060_07", "cl09_060_08",
+    "cl09_060_09", "cl09_060_10", "cl09_080_01", "cl09_080_02", "cl09_080_03",
+    "cl09_080_04", "cl09_080_05", "cl09_080_06", "cl09_080_07", "cl09_080_08",
+    "cl09_080_09", "cl09_080_10", "cl09_100_01", "cl09_100_02", "cl09_100_03",
+    "cl09_100_04", "cl09_100_05", "cl09_100_06", "cl09_100_07", "cl09_100_08",
+    "cl09_100_09", "cl09_100_10", "cl10_020_01", "cl10_020_02", "cl10_020_03",
+    "cl10_020_04", "cl10_020_05", "cl10_020_06", "cl10_020_07", "cl10_020_08",
+    "cl10_020_09", "cl10_020_10", "cl10_040_01", "cl10_040_02", "cl10_040_03",
+    "cl10_040_04", "cl10_040_05", "cl10_040_06", "cl10_040_07", "cl10_040_08",
+    "cl10_040_09", "cl10_040_10", "cl10_060_01", "cl10_060_02", "cl10_060_03",
+    "cl10_060_04", "cl10_060_05", "cl10_060_06", "cl10_060_07", "cl10_060_08",
+    "cl10_060_09", "cl10_060_10", "cl10_080_01", "cl10_080_02", "cl10_080_03",
+    "cl10_080_04", "cl10_080_05", "cl10_080_06", "cl10_080_07", "cl10_080_08",
+    "cl10_080_09", "cl10_080_10", "cl10_100_01", "cl10_100_02", "cl10_100_03",
+    "cl10_100_04", "cl10_100_05", "cl10_100_06", "cl10_100_07", "cl10_100_08",
+    "cl10_100_09", "cl10_100_10")
+
+
+def __divide_based_on_size(instances: Iterable[str],
+                           divisions: int) -> list[list[str]]:
+    """
+    Divide the instances based on their size.
+
+    :param instances: the instances
+    :param divisions: the number of divisions
+    :return: the divided instances
+    """
+    insts: list[str] = list(instances)
+    if len(insts) <= 0:
+        return [insts]
+
+    try:
+        loaded: list[Instance] = [Instance.from_resource(n) for n in insts]
+    except (OSError, ValueError):
+        return [insts]
+
+    # get the quantiles = group divisions
+    qants: list[float | int] = quantiles((
+        inst.n_items for inst in loaded), n=divisions, method="inclusive")
+
+    # now put the instances into the groups
+    groups: list[list[str]] = []
+    for inst in loaded:
+        inst_size = inst.n_items
+        idx: int = 0
+        for q in qants:
+            if q > inst_size:
+                break
+            idx += 1
+        while idx >= len(groups):
+            groups.append([])
+        groups[idx].append(str(inst))
+
+    # remove useless groups
+    for idx in range(len(groups) - 1, -1, -1):
+        if len(groups[idx]) <= 0:
+            del groups[idx]
+    return groups
+
+
+def _make_instance_groups(instances: Iterable[str]) \
+        -> tuple[tuple[str, str | None, tuple[str, ...]], ...]:
+    """
+    Make the standard instance groups from an instance name list.
+
+    :return: the instance groups
+    """
+    groups: list[tuple[str, str | None, tuple[str, ...]]] = []
+
+    a_divided: list[list[str]] = __divide_based_on_size(sorted(
+        b for b in instances if b.startswith("a")
+        and (len(b) == 3) and b[-1].isdigit()
+        and b[-2].isdigit()), 3)
+    if len(a_divided) > 0:
+        subnames: list[str | None] = [None] if (len(a_divided) <= 1) else (
+            ["small", "large"] if (len(a_divided) <= 2) else
+            ["small", "med", "large"])
+        for a_idx, a_group in enumerate(a_divided):
+            v: tuple[str, str | None, tuple[str, ...]] = (
+                "a", subnames[a_idx], tuple(a_group))
+            if len(v[2]) > 0:
+                groups.append(v)
+
+    v = ("beng", "1-8", tuple(sorted(
+        b for b in instances if b.startswith("beng")
+        and (int(b[-2:]) < 9))))
+    if len(v[2]) > 0:
+        groups.append(v)
+
+    v = ("beng", "9-10", tuple(sorted(
+        b for b in instances if b.startswith("beng")
+        and (int(b[-2:]) >= 9))))
+    if len(v[2]) > 0:
+        groups.append(v)
+
+    for i in range(1, 11):
+        name: str = f"class {i}"
+        preprefix: str = f"cl0{i}" if i < 10 else f"cl{i}"
+        for n in (20, 40, 60, 80, 100):
+            prefix: str = f"{preprefix}_0{n}_" \
+                if n < 100 else f"{preprefix}_{n}_"
+            v = (name, str(n), tuple(sorted(
+                b for b in _INSTANCES if b.startswith(prefix))))
+            if len(v[2]) > 0:
+                groups.append(v)
+
+    v = ("asqas", None, tuple(sorted(
+        b for b in _INSTANCES if b.startswith("asqas"))))
+    if len(v[2]) > 0:
+        groups.append(v)
+
+    all_set: set[str] = set()
+    for g in groups:
+        all_set.update(g[2])
+    inst_set: set[str] = set(instances)
+    if all_set != inst_set:
+        raise ValueError(f"group instances is {all_set!r} but "
+                         f"instance set is {inst_set!r}!")
+    return tuple(groups)
 
 
 def __cutsq(matrix: np.ndarray) -> list[int]:
@@ -387,8 +509,9 @@ def _lower_bound_damv(bin_width: int, bin_height: int,
     if bin_height > bin_width:
         bin_width, bin_height = bin_height, bin_width
     j_sq: Final[list[int]] = __cutsq(matrix)
-    return max(__lb_q(bin_width, bin_height, q, j_sq)
-               for q in range((bin_height // 2) + 1))
+    res: Final[int] = max(__lb_q(bin_width, bin_height, q, j_sq)
+                          for q in range((bin_height // 2) + 1))
+    return max(1, res)
 
 
 class Instance(Component, np.ndarray):
@@ -424,7 +547,8 @@ class Instance(Component, np.ndarray):
         :param name: the name of the instance
         :param bin_width: the bin width
         :param bin_height: the bin height
-        :param matrix: the matrix with the data (will be copied)
+        :param matrix: the matrix with the data (will be copied); rows have
+            the form `(width, height, repetitions)`
         """
         use_name: Final[str] = sanitize_name(name)
         if name != use_name:
@@ -558,12 +682,12 @@ class Instance(Component, np.ndarray):
         :param file: the file path
         :return: the instance
         """
-        path: Final[Path] = Path.file(file)
+        path: Final[Path] = file_path(file)
         name: str = basename(path).lower()
         if name.endswith(".ins2d"):
             name = sanitize_name(name[:-6])
 
-        lines: Final[list[str]] = path.read_all_list()
+        lines: Final[list[str]] = str.splitlines(path.read_all_str())
         n_different_items: Final[int] = check_to_int_range(
             lines[0], "n_different_items", 1, 100_000_000)
 
@@ -690,10 +814,69 @@ class Instance(Component, np.ndarray):
         :return: the tuple with the instance names
 
         >>> len(list(Instance.list_resources()))
-        553
+        557
         """
         return _INSTANCES
 
+    @staticmethod
+    def list_resources_groups() -> tuple[tuple[
+            str, str | None, tuple[str, ...]], ...]:
+        """
+        List the instance groups in the resources.
+
+        One problem of the benchmark set for 2-dimensional bin packing is that
+        it has many instances:
+
+        >>> len(Instance.list_resources())
+        557
+
+        With this function, we can group several of these instances together
+        in a way that is compliant with literature in order to then compute
+        statistics over these groups. Presenting data gathered over...
+
+        >>> len(Instance.list_resources_groups())
+        56
+
+        ...groups is much easier than dealing with over 500 instances.
+
+        :return: the instance groups, in a two level hierarchy. The result is
+            a sequence of tuples. Each tuple has the top-level group name and,
+            optionally, a second-level group name (or `None` if no second
+            level group exists). The third tuple element is a sequence of
+            instance names.
+
+        >>> [(v[0], v[1], len(v[2])) for v in
+        ...     Instance.list_resources_groups()]
+        [('a', 'small', 14), ('a', 'med', 14), ('a', 'large', 15), \
+('beng', '1-8', 8), ('beng', '9-10', 2), ('class 1', '20', 10), \
+('class 1', '40', 10), ('class 1', '60', 10), ('class 1', '80', 10), \
+('class 1', '100', 10), ('class 2', '20', 10), ('class 2', '40', 10), \
+('class 2', '60', 10), ('class 2', '80', 10), ('class 2', '100', 10), \
+('class 3', '20', 10), ('class 3', '40', 10), ('class 3', '60', 10), \
+('class 3', '80', 10), ('class 3', '100', 10), ('class 4', '20', 10), \
+('class 4', '40', 10), ('class 4', '60', 10), ('class 4', '80', 10), \
+('class 4', '100', 10), ('class 5', '20', 10), ('class 5', '40', 10), \
+('class 5', '60', 10), ('class 5', '80', 10), ('class 5', '100', 10), \
+('class 6', '20', 10), ('class 6', '40', 10), ('class 6', '60', 10), \
+('class 6', '80', 10), ('class 6', '100', 10), ('class 7', '20', 10), \
+('class 7', '40', 10), ('class 7', '60', 10), ('class 7', '80', 10), \
+('class 7', '100', 10), ('class 8', '20', 10), ('class 8', '40', 10), \
+('class 8', '60', 10), ('class 8', '80', 10), ('class 8', '100', 10), \
+('class 9', '20', 10), ('class 9', '40', 10), ('class 9', '60', 10), \
+('class 9', '80', 10), ('class 9', '100', 10), ('class 10', '20', 10), \
+('class 10', '40', 10), ('class 10', '60', 10), ('class 10', '80', 10), \
+('class 10', '100', 10), ('asqas', None, 4)]
+        """
+        obj: object = Instance.list_resources_groups
+        attr: str = "__gs"
+        if not hasattr(obj, attr):
+            gs: tuple[tuple[str, str | None, tuple[str, ...]], ...] =\
+                _make_instance_groups(Instance.list_resources())
+            setattr(obj, attr, gs)
+            return gs
+        return cast(tuple[tuple[str, str | None, tuple[str, ...]], ...],
+                    getattr(obj, attr))
+
     @staticmethod
     def from_resource(name: str) -> "Instance":
         """
@@ -724,8 +907,8 @@ class Instance(Component, np.ndarray):
         if hasattr(container, text_attr):  # ok, we got the text already
             text = cast(list[str], getattr(container, text_attr))
         else:  # the first time we load the text
-            with resources.open_text(package=str(__package__),
-                                     resource=INSTANCES_RESOURCE) as stream:
+            with resources.files(__package__).joinpath(
+                    INSTANCES_RESOURCE).open("r", encoding=UTF8) as stream:
                 text = [line for line in (line_raw.strip() for line_raw
                                           in stream.readlines())
                         if len(line) > 0]
