@@ -11,7 +11,9 @@ QAPLIB -- A Quadratic Assignment Problem Library.
 """
 
 from importlib import resources  # nosem
-from typing import TextIO
+from typing import TextIO, cast
+
+from pycommons.io.path import UTF8
 
 
 def open_resource_stream(file_name: str) -> TextIO:
@@ -21,4 +23,5 @@ def open_resource_stream(file_name: str) -> TextIO:
     :param file_name: the file name of the resource
     :return: the stream
     """
-    return resources.open_text(package=f"{__package__}", resource=file_name)
+    return cast(TextIO, resources.files(__package__).joinpath(
+        file_name).open("r", encoding=UTF8))
