@@ -74,7 +74,7 @@ import numba  # type: ignore
 import numpy as np
 from moptipy.api.objective import Objective
 from moptipy.utils.logger import KeyValueLogSection
-from moptipy.utils.types import type_error
+from pycommons.types import type_error
 
 from moptipyapps.shared import SCOPE_INSTANCE
 from moptipyapps.tsp.instance import Instance
