@@ -16,7 +16,9 @@ The original data of `robinX` can be found at
 """
 
 from importlib import resources  # nosem
-from typing import TextIO
+from typing import TextIO, cast
+
+from pycommons.io.path import UTF8
 
 
 def open_resource_stream(file_name: str) -> TextIO:
@@ -26,4 +28,5 @@ def open_resource_stream(file_name: str) -> TextIO:
     :param file_name: the file name of the resource
     :return: the stream
     """
-    return resources.open_text(package=f"{__package__}", resource=file_name)
+    return cast(TextIO, resources.files(__package__).joinpath(
+        file_name).open("r", encoding=UTF8))
